/**************************************************************************
** Copyright (c) 1996 Novell, Inc.  All Rights Reserved.
**
** THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND
** TREATIES.  USE AND REDISTRIBUTION OF THIS WORK IS SUBJECT TO THE
** LICENSE AGREEMENT ACCOMPANYING THE SOFTWARE DEVELOPMENT KIT (SDK)
** THAT CONTAINS THIS WORK.
** 
** Pursuant to the SDK License Agreement, Novell hereby grants to
** Developer a royalty-free, non-exclusive license to include the
** sample code SAPGENCB.C and derivative binaries in its product.
** Novell grants to Developer worldwide distribution rights to market,
** distribute or sell the sample code SAPGENCB.C and derivative
** binaries as a component of Developer's product(s).  Novell shall
** have no obligations to Developer or Developer's customers with
** respect to this code.
** 
** DISCLAIMER:
** 
** 	Novell, Inc. makes no representations or warranties with respect
** to the contents or use of this code, and specifically disclaims any
** express or implied warranties of merchantability or fitness for any
** particular purpose.  Further, Novell, Inc. reserves the right to revise
** this publication and to make changes to its content, at any time,
** without obligation to notify any person or entity of such revisions or
** changes.
**
** 	Further, Novell, Inc. makes no representations or warranties with
** respect to any software, and specifically disclaims any express or
** implied warranties of merchantability or fitness for any particular
** purpose.  Further, Novell, Inc. reserves the right to make changes to
** any and all parts of the software, at any time, without obligation to
** notify any person or entity of such changes.
**
***************************************************************************
**  File:   BOARD.C
**
**  Desc:   LAN board specific part of the DAI sample
**
**  Programmers:
**
**      Ini Who                     Firm
**      -----------------------------------------------------------------------
**      JB  Jan Beulich	            Novell Developer Support.
**
**  History:
**
**      When        Who What
**      -----------------------------------------------------------------------
**      10Jun95     jb  First code.
*/

#include <stdio.h>
#include <string.h>
#include <conio.h>
#include <errno.h>

#include "common.h"

#define BOARD_MODE_COUNT	11

static const FLAG2STRING boardModeTable[BOARD_MODE_COUNT] = {
	{MM_DMA_BIT,			"Uses DMA"},
	{MM_MULTICAST_BIT,		"Supports multicasting"},
	{MM_POINT_TO_POINT_BIT,		"Point-To-Point"},
	{MM_PREFILLED_ECB_BIT,		"Supports prefilled ECBs"},
	{MM_RAW_SENDS_BIT,		"Supports raw sends"},
	{MM_DATA_SZ_UNKNOWN_BIT,	"Can set LookAhead data size"},
	{MM_SMP_BIT,			"SMP aware"},
	{MM_FRAG_RECEIVES_BIT,		"Supports fragmented receives"},
	{MM_C_HSM_BIT,			"Written in C"},
	{MM_FRAGS_PHYS_BIT,		"Supports physical frags"},
	{MM_PROMISCUOUS_BIT,		"Supports promiscuous mode"}
};

#define BOARD_ATTR_COUNT	4

static const FLAG2STRING boardAttrTable[BOARD_ATTR_COUNT] = {
	{MF_EISA_BIT,			"EISA board"},
	{MF_ISA_BIT,			"ISA board"},
	{MF_MCA_BIT,			"MCA board"},
	{MF_HUB_MANAGEMENT_BIT,		"Supports hub management"}
};

void showBoardInfo(void*__info) {
#define info ((LAN_BOARD_INFO_STRUCT*)__info)

	unsigned i;
	int hScreen;
	unsigned line = 0;
	char s[128];

	sprintf(s, "%s %02X%02X%02X%02X%02X%02X",
		info->lanCardName,
		info->lanNodeAddress[0],
		info->lanNodeAddress[1],
		info->lanNodeAddress[2],
		info->lanNodeAddress[3],
		info->lanNodeAddress[4],
		info->lanNodeAddress[5]);
	hScreen = CreateScreen(s, POP_UP_SCREEN);
	if(hScreen == EFAILURE) return;
	SetCurrentScreen(hScreen);

	sprintf(s, "Short Name : %s", info->lanShortName);
	display(s, 0, 0, &line);

	if(info->lanSlot)
		i = sprintf(s, "Slot %hu ", info->lanSlot);
	else
		i = 0;
	i += sprintf(s+i, "Sharing Flags = %04hX ", info->lanSharingFlags);
	if(info->lanChannel)
		sprintf(s+i, "Channel %hu",
		info->lanChannel);
	display(s, 0, 0, &line);

	sprintf(s, "Driver Instance %u", info->lanBoardInstance);
	display(s, 0, 0, &line);

	showFlags(info->lanModeFlags, boardModeTable, BOARD_MODE_COUNT, &line);

	showFlags(info->lanFlags, boardAttrTable, BOARD_ATTR_COUNT, &line);

	for(i = 0; i < 2; i++) {
		if(info->lanPort[i].portBase) {
			sprintf(s, "I/O address range %u : %04hX-%04hX",
				i + 1,
				info->lanPort[i].portBase,
				info->lanPort[i].portBase + info->lanPort[i].portRange - 1);
			display(s, 0, 0, &line);
		}

		if(info->lanMemory[i].memBase) {
			sprintf(s, "Memory address range %u : %08lX-%08lX mapped ar %08X",
				i + 1,
				info->lanMemory[i].memBase,
				info->lanMemory[i].memBase + (info->lanMemory[i].memRange << 4) - 1,
				info->lanLinearMemory[i]);
			display(s, 0, 0, &line);
		}

		if(info->lanIntLine[i] != 0xFF) {
			sprintf(s, "Interrupt %u : %u",
				i + 1,
				info->lanIntLine[i]);
			display(s, 0, 0, &line);
		}

		if(info->lanDmaLine[i] != 0xFF) {
			sprintf(s, "DMA channel %u : %u",
				i + 1,
				info->lanDmaLine[i]);
			display(s, 0, 0, &line);
		}
	}

	sprintf(s, "Maximum Packet Size = %d bytes", info->lanMaximumSize);
	display(s, 0, 0, &line);

	pressAnyKeyToQuit();
	DestroyScreen(hScreen);
}
