/**************************************************************************
** Copyright (c) 1996 Novell, Inc.  All Rights Reserved.
**
** THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND
** TREATIES.  USE AND REDISTRIBUTION OF THIS WORK IS SUBJECT TO THE
** LICENSE AGREEMENT ACCOMPANYING THE SOFTWARE DEVELOPMENT KIT (SDK)
** THAT CONTAINS THIS WORK.
** 
** Pursuant to the SDK License Agreement, Novell hereby grants to
** Developer a royalty-free, non-exclusive license to include the
** sample code SAPGENCB.C and derivative binaries in its product.
** Novell grants to Developer worldwide distribution rights to market,
** distribute or sell the sample code SAPGENCB.C and derivative
** binaries as a component of Developer's product(s).  Novell shall
** have no obligations to Developer or Developer's customers with
** respect to this code.
** 
** DISCLAIMER:
** 
** 	Novell, Inc. makes no representations or warranties with respect
** to the contents or use of this code, and specifically disclaims any
** express or implied warranties of merchantability or fitness for any
** particular purpose.  Further, Novell, Inc. reserves the right to revise
** this publication and to make changes to its content, at any time,
** without obligation to notify any person or entity of such revisions or
** changes.
**
** 	Further, Novell, Inc. makes no representations or warranties with
** respect to any software, and specifically disclaims any express or
** implied warranties of merchantability or fitness for any particular
** purpose.  Further, Novell, Inc. reserves the right to make changes to
** any and all parts of the software, at any time, without obligation to
** notify any person or entity of such changes.
**
***************************************************************************
**  File:   DAI.C
**
**  Desc:   Main routines of the DAI sample
**
**  Programmers:
**
**      Ini Who                     Firm
**      -----------------------------------------------------------------------
**      JB  Jan Beulich	            Novell Developer Support.
**
**  History:
**
**      When        Who What
**      -----------------------------------------------------------------------
**      10Jun96     jb  First code.
*/

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <string.h>
#include <process.h>
#include <conio.h>
#include <advanced.h>
#include <nwdir.h>
#include <errno.h>

#include "common.h"

typedef struct INFO_DESCRIPTOR_TAG {
	void		*infoPtr;
	void		(*infoFunc)(void*);
} INFO_DESCRIPTOR;

static LONG downWarning(void(*)(void*,...), LONG);
static void downReport(LONG);

int main(/* int argc, char**argv */) {
	LONG		i,i1,j,j1,k,k1;
	unsigned	long n, itemCount;
	unsigned	line = 0;
	LONG		systemCount = 0, deviceCount = 0;
	LONG		partitionCount = 0, volumeCount = 0;
	unsigned	boardTypes = 0, boardCount = 0;
	unsigned	curDeviceCount, curPartitionCount;
	unsigned	curBoardCount, curFrameCount;
	void		*ssInfo;
	SYSTEM_INFO_STRUCT*systemInfo;
	DEVICE_INFO_STRUCT*deviceInfo;
	PARTITION_INFO_STRUCT*partitionInfo;
	MIRROR_INFO_STRUCT*mirrorInfo;
	REDIRECTION_INFO_STRUCT*redirInfo;
	VOLUME_STATS	*volumeInfo;
	LAN_BOARD_INFO_STRUCT*vboardInfo;
	INFO_DESCRIPTOR*infoTable;
	LAN_BOARD_LIST_STRUCT*boardList;
	LONG		eventID;
	char		*p;
	unsigned char	s[128];

	if(ReturnSystemCount(&systemCount) == NOERROR) {
		sprintf(s, "System Count = %ld", systemCount);
		display(s, 0, 0, &line);
	}

	if(ReturnDeviceCount(&deviceCount) == NOERROR) {
		sprintf(s, "Device Count = %ld", deviceCount);
		display(s, 0, 0, &line);
	}

	if(ReturnPartitionCount(&partitionCount) == NOERROR) {
		sprintf(s, "Partition Count = %ld", partitionCount);
		display(s, 0, 0, &line);
	}

	for(i = 0; GetVolumeName(i, s) == ESUCCESS; i++)
		if(*s)
			volumeCount++;

	if((boardList = malloc(SS_DEFAULT_BUFFER_SIZE)) == NULL) {
		puts("Not enough memory");
		return 1;
	}

	if(SSGetActiveLANBoardList(0, boardList, SS_DEFAULT_BUFFER_SIZE) != NOERROR)
		boardList->lblCount = 0;
	else {
		sprintf(s, "Board count = %ld", boardList->lblCount);
		display(s, 0, 0, &line);
	}

	display("", 0, 0, &line);

	if((systemInfo = malloc(systemCount*sizeof(SYSTEM_INFO_STRUCT)+
			deviceCount*sizeof(DEVICE_INFO_STRUCT)+
			partitionCount*sizeof(PARTITION_INFO_STRUCT)+
			partitionCount*sizeof(REDIRECTION_INFO_STRUCT)+
			partitionCount*sizeof(MIRROR_INFO_STRUCT)+
			boardList->lblCount*sizeof(LAN_BOARD_INFO_STRUCT)+
			volumeCount*sizeof(VOLUME_STATS)+
			SS_DEFAULT_BUFFER_SIZE)) == NULL) {
		puts("Not enough memory");
		free(boardList);
		return 1;
	}
	deviceInfo = (void*)(systemInfo + systemCount);
	partitionInfo = (void*)(deviceInfo + deviceCount);
	redirInfo = (void*)(partitionInfo + partitionCount);
	mirrorInfo = (void*)(redirInfo + partitionCount);
	vboardInfo = (void*)(mirrorInfo + partitionCount);
	volumeInfo = (void*)(vboardInfo + boardList->lblCount);
	ssInfo = (void*)(volumeInfo + volumeCount);

	if(SSGetCPUInfo(0, ssInfo, SS_DEFAULT_BUFFER_SIZE) != ESUCCESS) {
		ssInfo = NULL;
		puts("Unable to get CPU information");
	}

	for(i = 0; i < systemCount; i++)
		if(ReturnSystemInfo(i, systemInfo + i) != NOERROR) {
			printf("Unable to get info about system %ld\n", i);
			free(systemInfo);
			free(boardList);
			return 2;
		}
		else {
			convertString(systemInfo[i].sysName);
			convertString(systemInfo[i].sysDriverName);
			convertString(systemInfo[i].sysConfigInfo.ioLogicalName);
		}

	for(i = 0; i < deviceCount; i++)
		if(ReturnDeviceInfo(i, deviceInfo + i) != NOERROR) {
			printf("Unable to get info about device %ld\n", i);
			free(systemInfo);
			free(boardList);
			return 2;
		}
		else {
			convertString(deviceInfo[i].devName);
			convertString(deviceInfo[i].devRawName);
		}

	for(i = 0; i < partitionCount; i++) {
		if(ReturnPartitionInfo(i, partitionInfo + i) != NOERROR) {
			printf("Unable to get info about partition %ld\n", i);
			free(systemInfo);
			free(boardList);
			return 2;
		}
		else
			convertString(partitionInfo[i].partName);

		if(ReturnRedirectionInfo(i, redirInfo + i) != NOERROR)
			redirInfo[i].redirPartitionSize = 0;

		if(ReturnMirrorInfo(i, mirrorInfo + i) != NOERROR)
			mirrorInfo[i].mirrCount = 0;

		partitionInfo[i].partRedirInfo = (LONG)(redirInfo + i);
		partitionInfo[i].partMirrorInfo = (LONG)(mirrorInfo + i);
	}

	for(i = 0; i < volumeCount;)
		if(GetVolumeInformation(0, 
			(BYTE)i,
			sizeof(VOLUME_STATS),
			volumeInfo + i) == ESUCCESS)
			i++;

	for(i = 0; i < boardList->lblCount; i++) {
		if((j = SSGetLANConfiguration(boardList->lblBoardNumber[i],
			vboardInfo + i,
			sizeof(LAN_BOARD_INFO_STRUCT))) != ESUCCESS) {
			printf("Unable to get info about board %ld (%08X)\n", boardList->lblBoardNumber[i], j);
			free(systemInfo);
			free(boardList);
			return 2;
		}
		else {
			vboardInfo[i].lanCardName++;
			vboardInfo[i].lanShortName++;
			vboardInfo[i].lanMediaType++;

			/*lanBoardInstance is documented to never be zero,
			  but IPTUNNEL sets it to zero, so this is a
			  workaround hoping zero implies a single board*/
			if(vboardInfo[i].lanBoardInstance == 0)
				vboardInfo[i].lanBoardInstance++;
		}
	}
	for(i = 0; i < boardList->lblCount; i++) {
		for(j = 0; j < i; j++)
			if(!memcmp(vboardInfo[i].lanNodeAddress,
				vboardInfo[j].lanNodeAddress,
				sizeof(vboardInfo[i].lanNodeAddress)))
				break;
		if(j == i) {
			boardCount++;
			if(vboardInfo[i].lanBoardInstance == 1)
				boardTypes++;
		}
	}
	itemCount = systemCount + deviceCount + partitionCount + volumeCount \
		+ boardList->lblCount + boardCount;

	if((infoTable = malloc(itemCount*sizeof(INFO_DESCRIPTOR))) == NULL) {
		puts("Not enough memory");
		free(systemInfo);
		free(boardList);
		return 1;
	}

	n = 0;
	if(ssInfo != NULL) 
		sprintf(s, "OS (%u %s CPU(s) on %s)",
			((CPU_INFO_STRUCT*)ssInfo)->cpuCount,
			((CPU_INFO_STRUCT*)ssInfo)->cpuStrings,
			((CPU_INFO_STRUCT*)ssInfo)->cpuStrings
				+ strlen(((CPU_INFO_STRUCT*)ssInfo)->cpuStrings
					+ strlen(((CPU_INFO_STRUCT*)ssInfo)->cpuStrings) + 1)
				+ strlen(((CPU_INFO_STRUCT*)ssInfo)->cpuStrings) + 2);
		else
			strcpy(s, "OS");
	display(s, 0, itemCount, &line);

	for(i = 0; i < systemCount; i++) {
		infoTable[n].infoPtr = systemInfo + i;
		infoTable[n].infoFunc = showSystemInfo;
		sprintf(s,"%c%s",
			i == systemCount+volumeCount+boardTypes-1 ? TREE_END : TREE_NEW,
			systemInfo[i].sysName);
		display(s, ++n, itemCount, &line);

		for(j = 0, curDeviceCount = 0; j < deviceCount; j++)
			if(deviceInfo[j].devSystemNumber == i)
				curDeviceCount++;

		for(j = j1 = 0; j < deviceCount; j++)
			if(deviceInfo[j].devSystemNumber == i) {
				infoTable[n].infoPtr = deviceInfo + j;
				infoTable[n].infoFunc = showDeviceInfo;
				sprintf(s,"%c %c%s",
					i == systemCount+volumeCount+boardTypes-1 ? ' ' : TREE_NORMAL,
					++j1 == curDeviceCount ? TREE_END : TREE_NEW,
					deviceInfo[j].devName);
				display(s, ++n, itemCount, &line);

				for(k = 0, curPartitionCount = 0; k < partitionCount; k++)
					if(partitionInfo[k].partDeviceNumber == j)
						curPartitionCount++;

				for(k = k1 = 0; k < partitionCount; k++)
					if(partitionInfo[k].partDeviceNumber == j) {
						infoTable[n].infoPtr = partitionInfo + k;
						infoTable[n].infoFunc = showPartitionInfo;
						sprintf(s,"%c %c %c%s",
							i == systemCount+volumeCount+boardTypes-1 ? ' ' : TREE_NORMAL,
							j1 == curDeviceCount ? ' ' : TREE_NORMAL,
							++k1 == curPartitionCount ? TREE_END : TREE_NEW,
							partitionInfo[k].partName);
						display(s, ++n, itemCount, &line);
					}
			}
	}

	for(i = 0; i < volumeCount; i++) {
		infoTable[n].infoPtr = volumeInfo + i;
		infoTable[n].infoFunc = showVolumeInfo;
		sprintf(s, "%c%s",
			i == volumeCount+boardTypes-1 ? TREE_END : TREE_NEW,
			volumeInfo[i].volumeName);
		display(s, ++n, itemCount, &line);
		if(SSGetVolumeSegmentList(volumeInfo[i].volumeNumber,
			ssInfo,
			SS_DEFAULT_BUFFER_SIZE) == ESUCCESS)
			for(j = 0; j < ((VOLUME_SEGMENT_LIST_STRUCT*)ssInfo)->vslSegmentCount; j++) {
				sprintf(s, "%c %c %lu Mb starting at sector %lu on device # %u",
					i == volumeCount+boardTypes-1 ? ' ' : TREE_NORMAL,
					j == ((VOLUME_SEGMENT_LIST_STRUCT*)ssInfo)->vslSegmentCount-1 ? TREE_END : TREE_NEW,
					((VOLUME_SEGMENT_LIST_STRUCT*)ssInfo)->vslSegments[j].vsSize >> 11,
					((VOLUME_SEGMENT_LIST_STRUCT*)ssInfo)->vslSegments[j].vsOffset,
					((VOLUME_SEGMENT_LIST_STRUCT*)ssInfo)->vslSegments[j].vsDevice);
				display(s, 0, itemCount, &line);
			}
	}

	for(i = i1 = 0; i < boardList->lblCount; i++) {
		for(j = 0; j < i; j++)
			if(!memcmp(vboardInfo[i].lanNodeAddress,
				vboardInfo[j].lanNodeAddress,
				sizeof(vboardInfo[i].lanNodeAddress)))
				break;

		if(j == i && vboardInfo[i].lanBoardInstance == 1) {
			sprintf(s,"%c%s",
				++i1 == boardTypes ? TREE_END : TREE_NEW,
				vboardInfo[i].lanCardName);
			display(s, 0, itemCount, &line);

			for(j = curBoardCount = 0; j < boardList->lblCount; j++)
				if(!strcmp(vboardInfo[i].lanCardName,
					vboardInfo[j].lanCardName)) {
					for(k = 0; k < j; k++)
						if(!memcmp(vboardInfo[j].lanNodeAddress,
							vboardInfo[k].lanNodeAddress,
							sizeof(vboardInfo[j].lanNodeAddress)))
						break;
					if(k == j)
						curBoardCount++;
				}

			for(j = j1 = 0; j < boardList->lblCount; j++)
				if(!strcmp(vboardInfo[i].lanCardName,
					vboardInfo[j].lanCardName)) {
					for(k = 0; k < j; k++)
						if(!memcmp(vboardInfo[j].lanNodeAddress,
							vboardInfo[k].lanNodeAddress,
							sizeof(vboardInfo[j].lanNodeAddress)))
							break;
					if(k == j) {
						infoTable[n].infoPtr = vboardInfo + j;
						infoTable[n].infoFunc = showBoardInfo;
						sprintf(s,"%c %c%02X%02X%02X%02X%02X%02X",
							i1 == boardTypes ? ' ' : TREE_NORMAL,
							++j1 == curBoardCount ? TREE_END : TREE_NEW,
							vboardInfo[j].lanNodeAddress[0],
							vboardInfo[j].lanNodeAddress[1],
							vboardInfo[j].lanNodeAddress[2],
							vboardInfo[j].lanNodeAddress[3],
							vboardInfo[j].lanNodeAddress[4],
							vboardInfo[j].lanNodeAddress[5]);
						display(s, ++n, itemCount, &line);

						for(k = curFrameCount = 0; k < boardList->lblCount; k++)
							if(!memcmp(vboardInfo[j].lanNodeAddress,
								vboardInfo[k].lanNodeAddress,
								sizeof(vboardInfo[j].lanNodeAddress)))
								curFrameCount++;

						for(k = k1 = 0; k < boardList->lblCount; k++)
							if(!memcmp(vboardInfo[j].lanNodeAddress,
								vboardInfo[k].lanNodeAddress,
								sizeof(vboardInfo[j].lanNodeAddress))) {
								infoTable[n].infoPtr = vboardInfo + k;
								infoTable[n].infoFunc = showVBoardInfo;
								sprintf(s,"%c %c %c%s",
									i1 == boardTypes ? ' ' : TREE_NORMAL,
									j1 == curBoardCount ? ' ' : TREE_NORMAL,
									++k1 == curFrameCount ? TREE_END : TREE_NEW,
									vboardInfo[k].lanLogicalName);
								display(s, ++n, itemCount, &line);
							}
					}
				}
		}
	}

	do {
		printf("Select a number (0 to exit): ");
		if((n = strtoul(gets(s), &p, 10))
			&& n <= itemCount) {
				BeginThreadGroup(infoTable[n-1].infoFunc,
					NULL,
					0,
					infoTable[n-1].infoPtr);
			}
	} while(n);

	free(infoTable);
	free(systemInfo);
	free(boardList);

	if((eventID = RegisterForEvent(EVENT_DOWN_SERVER,
			downReport,
			downWarning)) != 0) {
		puts("You can now attempt to down the server or");
		pressAnyKeyToQuit();
		if(UnregisterForEvent/*Notification*/(eventID))
			puts("Failed to clean up event notification, server might be unstable");
	}
	else
		printf("Could not register for Server Down Notification");

	DestroyScreen(GetCurrentScreen());
	return 0;
}

static LONG downWarning(void(*output)(void*,...), LONG parm) {
	output("DAI Demo: Do not down the Server at this time!\n", parm);
	return -1;
}

static void downReport(LONG parm) {
	ConsolePrintf("Downing Server was attempted\n", parm);
}
