/**************************************************************************
** Copyright (c) 1996 Novell, Inc.  All Rights Reserved.
**
** THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND
** TREATIES.  USE AND REDISTRIBUTION OF THIS WORK IS SUBJECT TO THE
** LICENSE AGREEMENT ACCOMPANYING THE SOFTWARE DEVELOPMENT KIT (SDK)
** THAT CONTAINS THIS WORK.
** 
** Pursuant to the SDK License Agreement, Novell hereby grants to
** Developer a royalty-free, non-exclusive license to include the
** sample code SAPGENCB.C and derivative binaries in its product.
** Novell grants to Developer worldwide distribution rights to market,
** distribute or sell the sample code SAPGENCB.C and derivative
** binaries as a component of Developer's product(s).  Novell shall
** have no obligations to Developer or Developer's customers with
** respect to this code.
** 
** DISCLAIMER:
** 
** 	Novell, Inc. makes no representations or warranties with respect
** to the contents or use of this code, and specifically disclaims any
** express or implied warranties of merchantability or fitness for any
** particular purpose.  Further, Novell, Inc. reserves the right to revise
** this publication and to make changes to its content, at any time,
** without obligation to notify any person or entity of such revisions or
** changes.
**
** 	Further, Novell, Inc. makes no representations or warranties with
** respect to any software, and specifically disclaims any express or
** implied warranties of merchantability or fitness for any particular
** purpose.  Further, Novell, Inc. reserves the right to make changes to
** any and all parts of the software, at any time, without obligation to
** notify any person or entity of such changes.
**
***************************************************************************
**  File:   DAI.H
**
**  Desc:   Definitions for using DAI (Device Application Interface)
**
**	This is not a complete collection of all function, structures, and
**	definitions that can be used with DAI. Updates to this may or may
**	not happen in the future.
**
**  Programmers:
**
**      Ini Who                     Firm
**      -----------------------------------------------------------------------
**      JB  Jan Beulich	            Novell Developer Support.
**
**  History:
**
**      When        Who What
**      -----------------------------------------------------------------------
**      10Jun96     jb  First code.
*/

#ifndef __DAI_H
#define __DAI_H

#include <nwtypes.h>

/* Device Status Bits */

#define	ACTIVATED					0x01
#define	MOUNTED						0x02
#define	LOCKED						0x04
#define	PARTITIONED					0x08
#define	REMOTE						0x10
#define	BEINGDEACTIVATED				0x20
#define	DEVICEBUSY					0x40

/* Device Access Bits */

#define REMOVABLEBIT					0x01
#define READONLYBIT					0x02
#define WRITESEQUENTIALBIT				0x04
#define DUALPORTEDBIT					0x08
#define GETREQUESTONPOLLBIT				0x10
#define HOTFIXINHIBITBIT				0x20
#define MIRRORINHIBITBIT				0x40
#define	POLLIFNOTBUSYBIT				0x80

/* Device Types */

#define	HARDDISK					0
#define CDROM						1
#define WORM						2
#define TAPE						3
#define MAGOPTICAL					4

#define DEVICE_TYPE_COUNT				5

/* Partition Status Bits */

#define	OPERATIONAL					0x01
#define	REDIRECTED					0x04
#define	MIRRORED					0x08
#define	PARITYDISK					0x10
#define	CLONE						0x20
#define	ORPHAN						0x40
#define	BEINGUSED					0x80
#define	PARTITIONALERTED				0x100
#define	LOGICALPARTITIONALERTED				0x200

/* Remirror Status codes */

#define	REMIRRORREADY					0
#define	REMIRRORDONE					1
#define	REMIRRORWAITING					2

/* Mirror Status Bits */

#define	MIRRORGROUPINSYNCH				0x01
#define	MIRRORGROUPALLHERE				0x02
#define	MIRRORGROUPOPERATIONAL				0x04
#define	IAMINSYNCH					0x10
#define	NEEDTOFLUSHMIRRORINFO				0x20

/* IO FUNCTION CODES */

#define ELEVATOROFFBIT				0x80
#define PRIORITYBIT				0x40

#define	READREQUEST				0x00
#define	RANDOMREADREQUEST			0x00
#define	WRITEREQUEST				0x01
#define	RANDOMWRITEREQUEST			0x01
#define	WRITEONCEREQUEST			0x02
#define	SEQUENTIALREADREQUEST			(ELEVATOROFFBIT | 0x03)
#define	SEQUENTIALWRITEREQUEST			(ELEVATOROFFBIT | 0x04)
#define	RESETENDOFMEDIAREQUEST			(ELEVATOROFFBIT | 0x05)
#define	SINGLEFILEMARKSREQUEST			(ELEVATOROFFBIT | 0x06)
#define	CONSECUTIVEFILESMARKREQUEST		(ELEVATOROFFBIT | 0x07)
#define	SINGLESETMARKSREQUEST			(ELEVATOROFFBIT | 0x08)
#define	CONSECUTIVESETMARKREQUEST		(ELEVATOROFFBIT | 0x09)
#define RELATIVEDATABLOCKREQUEST		(ELEVATOROFFBIT | 0x0A)
#define	ABSOLUTEDATABLOCKREQUEST		(ELEVATOROFFBIT | 0x0B)
#define	SEQUENTIALPARTITIONREQUEST		(ELEVATOROFFBIT | 0x0C)
#define	PHYSICALMEDIAREQUEST			(ELEVATOROFFBIT | 0x0D)

/* IOCTL FUNCTION CODES */

#define	ACTIVATEDEVICE					0x00
#define	DEACTIVATEDEVICE				0x01
#define	FORMATDEVICE					0x02
#define	DEVICEVERIFYMODE				0x03
#define	DEVICEIDENTIFYSTATUS				0x04
#define	RETURNBADBLOCKINFO				0x05
#define	RETURNDEVICESTATUS				0x06
#define	LOGICALDEVICEMOUNT				0x07
#define	LOGICALDEVICEDISMOUNT				0x08
#define	LOCKDEVICEMEDIA					0x09
#define	UNLOCKDEVICEMEDIA				0x0A
#define	EJECTDEVICEMEDIA				0x0B
#define	RETURNCHANGERINFO				0x0C
#define	RETURNSLOTINFO					0x0D
#define	CHANGERCOMMAND					0x0E
#define	DEVICESELECTMEDIA				0x0F
#define	DEVICEUNSELECTMEDIA				0x10
#define	RETURNDEVICEINFO				0x11
#define	RETURNDEVICEMEDIAINFO				0x12
#define	SETDEVICEPARAMETERS				0x13

/* IO AND IOCTL COMPLETION CODES (low byte) */

#define NOERROR						0x00
#define CORRECTEDMEDIAERROR				0x01
#define MEDIAERROR					0x02
#define NONMEDIAERROR					0x03
#define NONMEDIAERRORFATAL				0x03
#define DEVICENOTACTIVE					0x04
#define ADAPTERERROR					0x05
#define	DEVICEPARAMETERERROR				0x06
#define	SYSTEMPARAMETERERROR				0x07
#define	NOTSUPPORTEDBYDEVICE				0x08
#define	NONMEDIAERRORNONFATAL				0x09
#define	NOTSUPPORTEDBYDRIVER				0xFFF9
#define	NOTSUPPORTED					0xFFF9

/* IO AND IOCTL EXCEPTION CODES (high byte) */

#define	UNDETERMINED					0x00
#define	DEVICEFAULT					0x01
#define	ENDOFMEDIA					0x02
#define	ENDOFFILE					0x03
#define	FILEMARKDETECTED				0x03
#define	ENDOFPARTITION					0x04
#define	EARLYWARNINGAREA				0x05
#define	MEDIACHANGE					0x06
#define	NOMEDIA						0x07
#define	NOMEDIAPRESENT					0x07
#define	WRITEPROTECTED					0x08
#define	SETMARKSDETECTED				0x09
#define	BLANKMEDIA					0x0A
#define	UNFORMATTEDMEDIA				0x0B
#define	DEVICENOTREADY					0x0C
#define	MEDIAPREVIOUSLYWRITTEN				0x0D
#define	ABORTEDDUETOPRIORSTATE				0x0E
#define	DESTINATIONOCCUPIED				0x0F

/* RESOURCE TAG SIGNATURES */

#define TimerSignature				0x524d4954
#define EventSignature				0x544e5645
#define DiskLogicalPartitionSignature		0x4c4b5344
#define DiskPartitionSignature			0x504b5344
#define DiskDeviceSignature			0x444b5344
#define DiskSystemSignature			0x534b5344


/* STRUCTURES */

typedef struct LOGICAL_PARTITION_INFO_TAG {
	WORD		logPartStatus;
	BYTE		logPartType;
	BYTE		logPartClass;
	LONG		logPartSize;
	LONG		logPartRawPartitionNumbers[16];
	BYTE		logPartName[64];
	LONG		logPartNotAvailable[8];
} LOGICAL_PARTITION_INFO_STRUCT;

typedef struct PARTITION_INFO_TAG {
	LONG		partDeviceNumber;
	WORD		partStatus;
	BYTE		partType;
	BYTE		partClass;
	LONG		partSize;
	LONG		partOffset;
	LONG		partIdentifier;
	LONG		partLogicalNumber;
	BYTE		partName[64];
	LONG		partNotAvailable;
} PARTITION_INFO_STRUCT;

typedef struct DEVICE_INFO_TAG {
	LONG		devPartition[4];
	LONG		devStatus;
	BYTE		devControllerNumber;
	BYTE		devDriveNumber;
	BYTE		devCardNumber;
	BYTE		devSystemType;
	BYTE		devAccessFlags;
	BYTE		devType;
	BYTE		devBlockSize;		/* 0=512 byte, 1 = 1 kb, etc. */
	BYTE		devSectorSize;		/* 0=512 byte, 1 = 1 kb, etc. */
	BYTE		devHeads;
	BYTE		devSectors;
	WORD		devCylinders;
	LONG		devCapacity;		/* number of sectors */
	LONG		devSystemNumber;
	LONG		devSystemID;
	BYTE		devRawName[40];
	BYTE		devName[64];
	LONG		devNotAvailable[8];
} DEVICE_INFO_STRUCT;

typedef struct IO_CONFIGURATION_TAG {
	LONG	ioNotAvailable1;
	WORD	ioSharingFlags;
	WORD	ioSlot;
	struct {
		WORD	portBase;
		WORD	portRange;
	} ioPort[2];
	struct {
		LONG	memBase;
		WORD	memRange;
	}	ioMemory[2];
	BYTE	ioIntLine[2];
	BYTE	ioDmaLine[2];
	LONG	ioNotAvailable2[2];
	BYTE	*ioCommandString;
	BYTE	ioLogicalName[18];
	BYTE	ioNotAvailable3[16];
} IO_CONFIGURATION_STRUCT;

typedef struct SYSTEM_INFO_TAG {
	BYTE	  	sysType;
	BYTE	  	sysProcessorNumber;
	WORD	  	sysUniqueTag;
	LONG	  	sysNumber;
	LONG		sysDevices[32];
	IO_CONFIGURATION_STRUCT	sysConfigInfo;
	BYTE		sysDriverName[36];
	BYTE		sysName[64];
	LONG		sysNumberOfDevices;
	LONG		sysNotAvailable[7];
} SYSTEM_INFO_STRUCT;

typedef struct REDIRECTION_INFO_TAG {
	LONG		redirPartitionSize;
	LONG		redirAreaSize;
	LONG		redirBlockCount;
	LONG		redirSystemBlockCount;
	LONG		redirReserved1;
} REDIRECTION_INFO_STRUCT;

typedef struct MIRROR_INFO_TAG {
	WORD		mirrStatus;
	WORD		mirrCount;
	LONG		mirrIdentifier;
	LONG		mirrMembers[16];
	LONG		mirrNotAvailable[8];
} MIRROR_INFO_STRUCT;


/* FUNCTIONS */

/*
LONG	RegisterForEventNotification(LONG, LONG, LONG,
		LONG(*)(void(*)(const char*,...),LONG), void(*)(LONG));
*/
LONG	ReturnDeviceCount(LONG*);
LONG	ReturnDeviceInfo(LONG, DEVICE_INFO_STRUCT*);
LONG	ReturnLogicalPartitionCount(LONG*);
LONG	ReturnLogicalPartitionInfo(LONG, LOGICAL_PARTITION_INFO_STRUCT*);
LONG	ReturnMirrorInfo(LONG, MIRROR_INFO_STRUCT*);
LONG	ReturnPartitionCount(LONG*);
LONG	ReturnPartitionInfo(LONG, PARTITION_INFO_STRUCT*);
LONG	ReturnRedirectionInfo(LONG, REDIRECTION_INFO_STRUCT*);
LONG	ReturnSystemCount(LONG*);
LONG	ReturnSystemInfo(LONG, SYSTEM_INFO_STRUCT*);
/*
LONG	UnRegisterEventNotification(LONG);
*/

#endif
