/**************************************************************************
** Copyright (c) 1996 Novell, Inc.  All Rights Reserved.
**
** THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND
** TREATIES.  USE AND REDISTRIBUTION OF THIS WORK IS SUBJECT TO THE
** LICENSE AGREEMENT ACCOMPANYING THE SOFTWARE DEVELOPMENT KIT (SDK)
** THAT CONTAINS THIS WORK.
** 
** Pursuant to the SDK License Agreement, Novell hereby grants to
** Developer a royalty-free, non-exclusive license to include the
** sample code SAPGENCB.C and derivative binaries in its product.
** Novell grants to Developer worldwide distribution rights to market,
** distribute or sell the sample code SAPGENCB.C and derivative
** binaries as a component of Developer's product(s).  Novell shall
** have no obligations to Developer or Developer's customers with
** respect to this code.
** 
** DISCLAIMER:
** 
** 	Novell, Inc. makes no representations or warranties with respect
** to the contents or use of this code, and specifically disclaims any
** express or implied warranties of merchantability or fitness for any
** particular purpose.  Further, Novell, Inc. reserves the right to revise
** this publication and to make changes to its content, at any time,
** without obligation to notify any person or entity of such revisions or
** changes.
**
** 	Further, Novell, Inc. makes no representations or warranties with
** respect to any software, and specifically disclaims any express or
** implied warranties of merchantability or fitness for any particular
** purpose.  Further, Novell, Inc. reserves the right to make changes to
** any and all parts of the software, at any time, without obligation to
** notify any person or entity of such changes.
**
***************************************************************************
**  File:   DEVICE.C
**
**  Desc:   Storage device specific part of the DAI sample
**
**  Programmers:
**
**      Ini Who                     Firm
**      -----------------------------------------------------------------------
**      JB  Jan Beulich	            Novell Developer Support.
**
**  History:
**
**      When        Who What
**      -----------------------------------------------------------------------
**      10Jun96     jb  First code.
*/

#include <stdio.h>
#include <string.h>
#include <conio.h>
#include <errno.h>

#include "common.h"

#define	DEVICE_STATUS_COUNT	7

static const FLAG2STRING devStatTable[DEVICE_STATUS_COUNT] = {
	{ACTIVATED,		"Activated"},
	{MOUNTED,		"Mounted"},
	{LOCKED,		"Locked"},
	{PARTITIONED,		"Partitioned"},
	{REMOTE,		"Remote"},
	{BEINGDEACTIVATED,	"Being deactivated"},
	{DEVICEBUSY,		"busy"}
};

#define DEVICE_ACCESS_COUNT	8

static const FLAG2STRING devAccessTable[DEVICE_ACCESS_COUNT] = {
	{REMOVABLEBIT,		"Removable"},
	{READONLYBIT,		"Read-Only"},
	{WRITESEQUENTIALBIT,	"Write-Sequential"},
	{DUALPORTEDBIT,		"Dual-Ported"},
	{GETREQUESTONPOLLBIT,	"Get request on poll"},
	{HOTFIXINHIBITBIT,	"Hot-Fix inhibited"},
	{MIRRORINHIBITBIT,	"Mirroring inhibited"},
	{POLLIFNOTBUSYBIT,	"Poll if not busy"}
};

static const char devTypeUnknown[] = "Unknown";

static const char*const devTypeTable[DEVICE_TYPE_COUNT] = {
	"Hard Disk",
	"CD-ROM",
	"WORM",
	"Tape",
	"Magneto-Optical"
};

void showDeviceInfo(void*__info) {
#define info ((DEVICE_INFO_STRUCT*)__info)
	int hScreen = CreateScreen(info->devName, POP_UP_SCREEN);
	unsigned line = 0;
	char s[128];

	if(hScreen == EFAILURE) return;
	SetCurrentScreen(hScreen);

	sprintf(s, "Raw Name : %s", info->devRawName);
	display(s, 0, 0, &line);

	showFlags(info->devStatus, devStatTable, DEVICE_STATUS_COUNT, &line);

	showFlags(info->devAccessFlags, devAccessTable, DEVICE_ACCESS_COUNT, &line);

	sprintf(s, "Drive Type : %s",
		info->devType < DEVICE_TYPE_COUNT ?
		devTypeTable[info->devType] :
		devTypeUnknown);
	display(s, 0, 0, &line);

	sprintf(s, "Controller Number = %u", info->devControllerNumber);
	display(s, 0, 0, &line);

	sprintf(s, "Drive Number = %u", info->devDriveNumber);
	display(s, 0, 0, &line);

	sprintf(s, "Card Number = %u", info->devCardNumber);
	display(s, 0, 0, &line);

	sprintf(s, "Sector Size = %lu bytes", 1UL << (9+info->devSectorSize));
	display(s, 0, 0, &line);

	sprintf(s, "Block Size = %lu Sectors (%lu bytes)",
		1UL << info->devBlockSize,
		(1UL << info->devBlockSize) * (1UL << (9+info->devSectorSize)));
	display(s, 0, 0, &line);

	if(info->devHeads != 0xFF
		|| info->devSectors != 0xFF
		|| info->devCylinders != 0xFFFF) {
		sprintf(s, "%u Heads, %u Cylinders, %u Sectors per Track",
			info->devHeads,
			info->devCylinders,
			info->devSectors);
		display(s, 0, 0, &line);

		sprintf(s, "Total Sectors = %lu, Total Space %lu Mbyte",
			info->devCapacity,
			info->devCapacity * (1UL << info->devSectorSize) >> 11);
		display(s, 0, 0, &line);

		if(info->devHeads * info->devCylinders * info->devSectors != info->devCapacity)
			display("Total Sectors do not match Heads*Cylinders*SPT", 0, 0, &line);
	}

	pressAnyKeyToQuit();
	DestroyScreen(hScreen);
}
