/**************************************************************************
** Copyright (c) 1996 Novell, Inc.  All Rights Reserved.
**
** THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND
** TREATIES.  USE AND REDISTRIBUTION OF THIS WORK IS SUBJECT TO THE
** LICENSE AGREEMENT ACCOMPANYING THE SOFTWARE DEVELOPMENT KIT (SDK)
** THAT CONTAINS THIS WORK.
** 
** Pursuant to the SDK License Agreement, Novell hereby grants to
** Developer a royalty-free, non-exclusive license to include the
** sample code SAPGENCB.C and derivative binaries in its product.
** Novell grants to Developer worldwide distribution rights to market,
** distribute or sell the sample code SAPGENCB.C and derivative
** binaries as a component of Developer's product(s).  Novell shall
** have no obligations to Developer or Developer's customers with
** respect to this code.
** 
** DISCLAIMER:
** 
** 	Novell, Inc. makes no representations or warranties with respect
** to the contents or use of this code, and specifically disclaims any
** express or implied warranties of merchantability or fitness for any
** particular purpose.  Further, Novell, Inc. reserves the right to revise
** this publication and to make changes to its content, at any time,
** without obligation to notify any person or entity of such revisions or
** changes.
**
** 	Further, Novell, Inc. makes no representations or warranties with
** respect to any software, and specifically disclaims any express or
** implied warranties of merchantability or fitness for any particular
** purpose.  Further, Novell, Inc. reserves the right to make changes to
** any and all parts of the software, at any time, without obligation to
** notify any person or entity of such changes.
**
***************************************************************************
**  File:   PART.C
**
**  Desc:   Partition specific part of the DAI sample
**
**  Programmers:
**
**      Ini Who                     Firm
**      -----------------------------------------------------------------------
**      JB  Jan Beulich	            Novell Developer Support.
**
**  History:
**
**      When        Who What
**      -----------------------------------------------------------------------
**      10Jun96     jb  First code.
*/

#include <stdio.h>
#include <conio.h>
#include <errno.h>

#include "common.h"

#define PARTITION_STATUS_COUNT	9

static const FLAG2STRING partStatTable[PARTITION_STATUS_COUNT] = {
	{OPERATIONAL,			"Operational"},
	{REDIRECTED,			"Redirected"},
	{MIRRORED,			"Mirrored"},
	{PARITYDISK,			"Parity disk"},
	{CLONE,				"Clone"},
	{ORPHAN,			"Orphan"},
	{BEINGUSED,			"Being used"},
	{PARTITIONALERTED,		"Partition alerted"},
	{LOGICALPARTITIONALERTED,	"Logical partition alerted"}
};

#define	MIRROR_STATUS_COUNT	5

static const FLAG2STRING mirrorStatTable[MIRROR_STATUS_COUNT] = {
	{MIRRORGROUPINSYNCH,		"Group in sync"},
	{MIRRORGROUPALLHERE,		"Group here"},
	{MIRRORGROUPOPERATIONAL,	"Group OK"},
	{IAMINSYNCH,			"I am in sync"},
	{NEEDTOFLUSHMIRRORINFO,		"Needs flush"}
};

static const struct {
	unsigned char type;
	char*name;
} partTypeTable[] = {
	{0x01,	"DOS12"},
	{0x04,	"DOS16"},
	{0x05,	"EXTENDED"},
	{0x06,	"BIGDOS"},
	{0x07,	"HPFS"},
	{0x0a,	"BOOTMGR"},
	{0x64,	"NW286"},
	{0x65,	"NW386"},
	{0x00,	"OTHER"}
};

void showPartitionInfo(void*__info) {
#define info ((PARTITION_INFO_STRUCT*)__info)
	unsigned i;
	int hScreen = CreateScreen(info->partName, POP_UP_SCREEN);
	unsigned line = 0;
	REDIRECTION_INFO_STRUCT*redirInfo;
	MIRROR_INFO_STRUCT*mirrorInfo;
	char s[128];

	if(hScreen == EFAILURE) return;
	SetCurrentScreen(hScreen);

	showFlags(info->partStatus, partStatTable, PARTITION_STATUS_COUNT, &line);

	for(i = 0; partTypeTable[i].type && partTypeTable[i].type != info->partType; i++)
		;

	sprintf(s, "Partition Type = %02X (%s), Partition Class = %02X",
		info->partType,
		partTypeTable[i].name,
		info->partClass);
	display(s, 0, 0, &line);

	sprintf(s, "%lu Sectors (%lu Mb) Starting at Sector %lu",
		info->partSize,
		info->partSize >> 11,
		info->partOffset);
	display(s, 0, 0, &line);

	if((redirInfo = (void*)(info->partRedirInfo))->redirPartitionSize) {
		display("", 0, 0, &line);
		display("Redirection Info:", 0, 0, &line);

		sprintf(s, "Partition Size : %lu Sectors (%lu Mb)",
			redirInfo->redirPartitionSize,
			redirInfo->redirPartitionSize >> 11);
		display(s, 0, 0, &line);

		sprintf(s, "Redirection Area Size : %lu Sectors (%lu kb)",
			redirInfo->redirAreaSize,
			redirInfo->redirAreaSize >> 1);
		display(s, 0, 0, &line);

		sprintf(s, "%lu System 4K Blocks", redirInfo->redirSystemBlockCount);
		display(s, 0, 0, &line);

		sprintf(s, "%lu Redirected 4K Blocks", redirInfo->redirBlockCount);
		display(s, 0, 0, &line);
	}

	if((mirrorInfo = (void*)(info->partMirrorInfo))->mirrCount) {
		display("", 0, 0, &line);
		display("Mirror Info:", 0, 0, &line);

		sprintf(s, "%hu Active Mirrors", mirrorInfo->mirrCount);
		display(s, 0, 0, &line);

		showFlags(mirrorInfo->mirrStatus, mirrorStatTable, MIRROR_STATUS_COUNT, &line);
	}

	pressAnyKeyToQuit();
	DestroyScreen(hScreen);
}
